package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.COMM;

import java.io.Serializable;

import tsp.csc4509.spool.Information;

public class ProtocolMessage implements Serializable {
	private static final long serialVersionUID = 1L;
	private ProtocolMessageType messageType;
	private Information information;
//	private String to;
//	private String from;
//	private String fileName;
	private int port;

	/**
	 * Constructeur pour les messages circulant du client vers le serveur (PUT et
	 * GET)
	 * 
	 * @param messageType type du message.
	 * @param information informations sur le fichier.
	 */
	public ProtocolMessage(final ProtocolMessageType messageType, final Information informattion) {
		super();
		this.messageType = messageType;
		this.information = informattion;
		this.port = 0;
	}

	/**
	 * Constructeur pour les messages circulant du serveur vers le client (SEND et
	 * RECEIVE)
	 * 
	 * @param messageType type du message.
	 * @param information informations sur le fichier.
	 * @param port        port préparé par serveur pour la connexion du client.
	 */
	public ProtocolMessage(final ProtocolMessageType messageType, final Information information, final int port) {
		super();
		this.messageType = messageType;
		this.information = information;
		this.port = port;

	}

	public ProtocolMessageType getMessageType() {
		return messageType;
	}

	public String getTo() {
		return information.getTo();
	}

	public String getFrom() {
		return information.getFrom();
	}

	public String getFileName() {
		return information.getFileName();
	}

	public int getPort() {
		return port;
	}

	public Information getInformation() {
		return information;
	}

	@Override
	public String toString() {
		return "ProtocolMessage [messageType=" + messageType + ", to=" + information.getTo() + ", from="
				+ information.getFrom() + ", fileName=" + information.getFileName() + ", port=" + port + "]";
	}

}
