package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.GEN;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.file.Path;

import tsp.csc4509.common.Log;
import tsp.csc4509.spool.Spool;

public class Server {

	/**
	 * répertoire du spool pour le serveur.
	 */
	private static final String SERVERSPOOLDIR = "data/serverSpool";

	/**
	 * le canal gérant les accepts du serveur.
	 */
	private ServerSocketChannel listenChannel;

	/**
	 * Constructeur de Server qui ouvre le canal du serveur, le lie au port passé en
	 * paramètre et initialise le spool ou sont mémorisés les fichiers coté serveur.
	 * 
	 * @param port port que doit occuper le serveur.
	 * @throws IOException toutes les exceptions qui peuvent se produire à la
	 *                     creation de la socket serveur.
	 */
	public Server(final int port) throws IOException {
		listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
		Spool.initSpool(SERVERSPOOLDIR);
	//	Log.COMM.info("Serveur créé");
	}

	/**
	 * Accepte la connexion d'un client sur le canal listenChannel.
	 * 
	 * @return le canal relié au client qui vient de se connecter.
	 * @throws IOException toutes les exceptions qui peuvent se produire à
	 *                     l'établissement de la connexion, et l'initialisation du
	 *                     spool du serveur.
	 */
	public SocketChannel accept() throws IOException {
		GEN.info("Server::accept()");

		return listenChannel.accept();
	}

	/**
	 * Lance le service du serveur. Boucle qui 1. accepte un client; 2. lance un
	 * thread pour spécifique pour recevoir et traiter le ProtocolMessage de ce
	 * client.
	 * 
	 * @throws IOException          Exceptions levées lors de l'établissement de la
	 *                              connexion.
	 * @throws InterruptedException Exceptions levées par la gestion du thread.
	 */
	public void service() throws IOException, InterruptedException { // le service n'est pas robuste: il ne récupère pas
																		// des exceptions
		GEN.info("Server::service() début");

		while (true) {
			SocketChannel clientChan = accept();
			GEN.info("Server::service() ajout d'un client en file d'attente");
			new Thread(new ServerProtocolInterpreterRunnable(clientChan)).start();
		}
	}

	public static void deleteSpoolDir(final Path path) throws Exception {
		Spool.deletePath(path, SERVERSPOOLDIR);
	}

}
