package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.COMM;
import static tsp.csc4509.common.Log.GEN;

import java.io.IOException;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.FutureTask;

import tsp.csc4509.spool.Spool;
import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;

/**
 * Runnable qui fournit le code du thread qui prend en charge les connexions de
 * chaque client (et leurs requêtes PUT et GET).
 * 
 * @author
 *
 */
public class ServerProtocolInterpreterRunnable implements Runnable {
	private final String badmessage = "Bad message received from client.";
	/**
	 * worker utilisé pour les échanges des messages du protocole.
	 */
	private FullDuplexMsgWorker worker;

	/**
	 * Contructeur qui initialise le worker à partir du canal connecté au client.
	 * 
	 * @param rwChan
	 */
	public ServerProtocolInterpreterRunnable(final SocketChannel rwChan) {
		super();
		this.worker = new FullDuplexMsgWorker(rwChan);
	}

	/**
	 * Réception d'un message du protocole reçu sur le worker connecté au client.
	 * 
	 * @return le message du protocole reçu.
	 * @throws IOException les exceptions levées lors de la réception du message.
	 */
	private ProtocolMessage receiveMessage() throws IOException {
		ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) {
			status = worker.readMessage();
		}
		if (status == ReadMessageStatus.ReadDataCompleted) {
			Serializable obj = worker.getData();
			if (obj instanceof ProtocolMessage pm) {
				return pm;
			}
		}
		return null;
	}

	// Q4
	/**
	 * Création d'un ServerSocketChannel sur port libre laissé au choix du système.
	 * 
	 * @return le ServerSocketChannel prêt à accepter les connexions.
	 * @throws IOException les exceptions levées lors de l'ouverture et le lien du
	 *                     ServerSocketChannel.
	 */
	public ServerSocketChannel createServerSocketChannel() throws IOException {// devrait private. Rendu public pour le
																				// test unitaire.

		// TODO Question 4
		return null; // pensez à changer le return
	}

	// Q5
	/**
	 * Prise en charge d'une requête PUT envoyée par un client.
	 * 
	 * @param putMessage Le message de protocole contenant la requête PUT.
	 * @throws Exception Toutes les exceptions levées pour gérer la requête.
	 */
	private void put(final ProtocolMessage putMessage) throws Exception {
		// création du répertoire pour recevoir le fichier dans le spool du serveur.
		Path path = Spool.initNewPath(putMessage);
		// création ServerSocketChannel lié à un port libre choisi par le système.
		ServerSocketChannel listenChannel = createServerSocketChannel();
		// on retrouve le port choisi par le système.
		int port = ((InetSocketAddress) listenChannel.getLocalAddress()).getPort();

		// TODO Question 5
	}

	// Q6
	/**
	 * Prise en charge d'une requête GET envoyée par un client.
	 * 
	 * @param getMessage Le message de protocole contenant la requête GET.
	 * @throws Exception Toutes les exceptions levées pour gérer la requête.
	 */
	private void get(final ProtocolMessage getMessage) throws Exception {
		// on recherche tout les fichiers dont le destinataire (champ To) est le From
		// contenu dans le getMessage.
		List<Path> pathList = Spool.getFilesListTo(getMessage.getFrom());

		
		// TODO Question 6
	}

	@Override
	public void run() {
		try {
			// réception du message de protocole envoyé par le client.
			ProtocolMessage message = receiveMessage();
			if (message == null) {
				COMM.warn(badmessage);
				return;
			}
			COMM.info("Reception de protocolMessage : {}", message);
			switch (message.getMessageType()) {
			case PUT: {
				// gestion des requêtes PUT.
				put(message);
				break;
			}
			case GET: {
				// gestion des requêtes GET.
				get(message);
			}
			default:
				// ne devrait pas se produire
			}
			worker.close(); // une fois la requête traitée, la connexion avec le client doit être fermée.
			// sinon le serveur va accumuler les connexions ouvertes et finira par saturer.
			// De plus le client ne saura jamais que son service est fini.
		} catch (Exception e) {
			// pas de récupération ni de traitement des exceptions dans cette version.
			e.printStackTrace();
		}
	}
}
