package tsp.csc4509.tpnote;

import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.file.Path;
import java.util.Random;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

class TestReceiver {
	/**
	 * numéro du port utilisé par les serveur lancés pour les tests.
	 */
	private static final int TESTSERVERPORT = 40001;

	/**
	 * taille des ByteBuffer.
	 */
	private static final int BUFFERSIZE = 1024;

	private static final String TESTSENDFILENAME = "sendfile.data";
	private static final String TESTRECVFILENAME = "recvfile.data";

	private static final int TESTFILESIZE = (int) (BUFFERSIZE * 2.5);

	@Disabled
	@Test
	void testRecvFile(@TempDir Path tempDir) throws Exception {
		final String SENDFILENAME = tempDir.resolve(TESTSENDFILENAME).toString();
		final String RECVFILENAME = tempDir.resolve(TESTRECVFILENAME).toString();
		// création d'un fichier au contenu aléatoire, et contenu du fichier stocké dans
		// bufferFile
		final ByteBuffer fileBuffer = createDataTestFile(SENDFILENAME, 0, TESTFILESIZE);
		// lancement du thread qui contient le serveur qui va envoyer les données au
		// recvFile().
		new Thread(new Runnable() {
			@Override
			public void run() {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						SocketChannel rwChan = listenChan.accept();) {

					rwChan.write(fileBuffer);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}

		}).start();
		Thread.sleep(1000);
		// création d'une instance de SenderReceiver
		SenderReceiver sender = new SenderReceiver() {
		};
		FileOutputStream fout = new FileOutputStream(RECVFILENAME);
		FileChannel fcout = fout.getChannel();
		InetSocketAddress servAddress = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);
		SocketChannel rwChan = SocketChannel.open(servAddress);
		// test de la méthode sendFile
		sender.recvFile(fcout, rwChan);
		fout.close();
		fcout.close();
		rwChan.close();
		assertTrue(compareFile(SENDFILENAME, RECVFILENAME), "Données reçues = données envoyées");
	}

	/**
	 * Création d'un ServerSocketChannel lié au port passé en paramètre.
	 * 
	 * @param port numéro du port du ServerSocketChannel.
	 * @return ServerSocketChannel lié au port.
	 * @throws IOException toutes les exceptions pouvant se produire lors de la
	 *                     création et du bind du ServerSocketChannel.
	 */
	private ServerSocketChannel createServerSocketChannel(final int port) throws IOException {
		ServerSocketChannel listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
		return listenChannel;
	}

	/**
	 * Compare le contenu de deux fichiers et retourne true s'ils sont identiques.
	 * 
	 * @param fileName1 nom du premier fichier.
	 * @param fileName2 nom du second fichier.
	 * @return true si les deux fichiers ont le même contenu, false sinon.
	 * @throws Exception toutes les exceptions possibles.
	 */
	private static boolean compareFile(final String fileName1, final String fileName2) throws Exception {
		try (FileInputStream fin1 = new FileInputStream(fileName1);
				FileInputStream fin2 = new FileInputStream(fileName2);
				FileChannel fcin1 = fin1.getChannel();
				FileChannel fcin2 = fin2.getChannel();) {
			ByteBuffer buffer1 = ByteBuffer.allocate(BUFFERSIZE);
			ByteBuffer buffer2 = ByteBuffer.allocate(BUFFERSIZE);
			int lu1, lu2;
			do {
				buffer1.clear();
				buffer2.clear();
				lu1 = fcin1.read(buffer1);
				lu2 = fcin2.read(buffer2);
				if (lu1 != lu2) {
					return false;
				}
				buffer1.flip();
				buffer2.flip();
				for (int ind = 0; ind < lu1; ind++) {
					if (buffer1.get() != buffer2.get()) {
						return false;
					}
				}

			} while (lu1 == BUFFERSIZE);
			return true;
		} // les close des fichiers sont automatiques avec la syntaxe try-with-resources
	}

	/**
	 * Remplit un fichier de bytes aléatoires. Le fichier est plus grand que les
	 * ByteBuffer utilisés pour les tests afin d'être sûr que plusieurs tours de
	 * boucles soient nécessaires pour le lire.
	 * 
	 * @param fileName   nom du fichier à remplir.
	 * @param randomSeed graine du générateur de la série des nombres
	 *                   pseudo-aléatoires mis dans le fichier.
	 * @return un ByteBuffer contenant toutes les données qui ont été écrites sur le
	 *         fichier.
	 */
	private static ByteBuffer createDataTestFile(final String fileName, final int randomSeed, final int dataSize) {
		byte[] randomBytes = new byte[dataSize];
		// générateur de nombres pseudo-aléatoires. Attention pour rendre les
		// tests reproductibles, il faut fixer le début de la suite pour régénérer
		// le même fichier à chaque lancement des tests.
		Random random = new Random(randomSeed);
		// tableau remplis d'octets aléatoires
		random.nextBytes(randomBytes);
		ByteBuffer buffer = ByteBuffer.wrap(randomBytes);
		// après un wrap() le curseur position est déjà à 0. Pas de flip() à faire.
		try (FileOutputStream fout = new FileOutputStream(fileName); FileChannel fcout = fout.getChannel();) {
			fcout.write(buffer);
		} catch (FileNotFoundException fnfe) {
			System.out.println(fnfe);
		} catch (IOException ioe) {
			System.out.println(ioe);
		}
		// le close des fichiers est automatique avec la syntaxe try () { }
		buffer.rewind();
		return buffer;
	}
}
